//+-----------------------------------------------------------------------------------------------+
//|                                                                                 MySqlTest.mq4 |
//|                                                                                   Zloy Koldun |
//+-----------------------------------------------------------------------------------------------+
#property copyright "Zloy Koldun"
#property version   "1.00"
#property strict

#include <ComObject.mqh>

enum CursorTypeEnum
{
   adOpenUnspecified = -1,
   adOpenForwardOnly = 0,
   adOpenKeyset      = 1,
   adOpenDynamic     = 2,
   adOpenStatic      = 3
};

enum LockTypeEnum
{
   adLockUnspecified     = -1,
   adLockReadOnly        = 1,
   adLockPessimistic     = 2,
   adLockOptimistic      = 3,
   adLockBatchOptimistic = 4
};

enum CommandTypeEnum
{
   adCmdUnspecified  = -1,
   adCmdText         = 1,
   adCmdTable        = 2,
   adCmdStoredProc   = 4,
   adCmdUnknown      = 8,
   adCmdFile         = 256,
   adCmdTableDirect  = 512
};

enum ExecuteOptionEnum
{
   adOptionUnspecified     = -1,
   adAsyncExecute          = 0x10,
   adAsyncFetch            = 0x20,
   adAsyncFetchNonBlocking = 0x40,
   adExecuteNoRecords      = 0x80,
   adExecuteStream         = 0x400
};

input string Server = "localhost";
input string Database = "test";

string Driver = "{MySQL ODBC 5.3 Unicode Driver}";
string Uid = "root";
string Password = "MySQL";
uint Port = 3306;

static const Variant varTime( "Time" );
static const Variant varAsk( "Ask" );
static const Variant varBid( "Bid" );
static const Variant varLast( "Last" );
static const Variant varTime_msc( "Time_msc" );
static const Variant varFlags( "Flags" );

static string mySymbol = Symbol();
static ComObject* Connection = NULL;
static ComObject* Recordset = NULL;
//+-----------------------------------------------------------------------------------------------+
//| Expert initialization function                                                                |
//+-----------------------------------------------------------------------------------------------+
int OnInit()
{
#ifdef _WIN64
   if( !_IsX64 ){
      Alert( "The program only works in a 64-bit terminal!" );
      return INIT_FAILED;
   }
#else
   if( _IsX64 ){
      Alert( "The program only works in a 32-bit terminal!" );
      return INIT_FAILED;
   }
#endif

   HRESULT result = CoInitialize( 0 );
   if( result != S_OK ){
      return INIT_FAILED;
   }
   
   Connection = new ComObject( "ADODB.Connection" );
   if( !Connection.isValid() ){
      return INIT_FAILED;
   }
   
   string ConnectionString = "Driver=" + Driver + ";Server=" + Server + ";Port=" + IntegerToString( Port ) + ";Option=4;Database=" + Database + ";Uid=" + Uid + ";Pwd=" + Password + ";";
   Variant varResult = Connection.Function( "Open", Variant( ConnectionString ) );
   if( !varResult.isValid() ){
      PrintError( *Connection );
      return INIT_FAILED;
   }
   
   varResult = Connection.Function( "Execute", Variant( "drop table " + mySymbol ) );
   if( !varResult.isValid() ){
      PrintError( *Connection );
   }
   
   string sql = "create table " + mySymbol + "( Time DATETIME, Ask double, Bid double, Last double, Time_msc bigint, Flags int )";
   varResult = Connection.Function( "Execute", Variant( sql ) );
   if( !varResult.isValid() ){
      PrintError( *Connection );
      return INIT_FAILED;
   }
   
   Recordset = new ComObject( "ADODB.Recordset" );
   if( !Recordset.isValid() ){
      return INIT_FAILED;
   }
   
   varResult = Recordset.Function( "Open", Variant( mySymbol ), *Connection, Variant( (int)adOpenForwardOnly ), Variant( (int)adLockOptimistic ), Variant( (int)adCmdTableDirect ) );
   if( !varResult.isValid() ){
      PrintError( *Connection );
      return INIT_FAILED;
   }
   
   return INIT_SUCCEEDED;
}
//+-----------------------------------------------------------------------------------------------+
//| Expert deinitialization function                                                              |
//+-----------------------------------------------------------------------------------------------+
void OnDeinit( const int reason )
{
   if( CheckPointer( Recordset ) == POINTER_DYNAMIC ){
      if( Recordset.isValid() ){
         Recordset.Function( "Close" );
      }
      delete Recordset;
   }
   if( CheckPointer( Connection ) == POINTER_DYNAMIC ){
      if( Connection.isValid() ){
         Connection.Function( "Close" );
      }
      delete Connection;
   }
   CoUninitialize();
}
//+-----------------------------------------------------------------------------------------------+
//| Expert tick function                                                                          |
//+-----------------------------------------------------------------------------------------------+
void OnTick()
{
   MqlTick tick;
   if( SymbolInfoTick( mySymbol, tick ) ){
      Variant varResult = Recordset.Function( "AddNew" );
      if( !varResult.isValid() ){
         Print( varResult.toErrorString() );
         return;
      }
      
      ComObject Fields = Recordset.GetProperty( "Fields" );
      if( Fields.isValid() ){
         ComObject Field = Fields.GetProperty( "Item", varTime );
         Field.SetProperty( "Value", Variant( tick.time ) );
         
         Field = Fields.GetProperty( "Item", varAsk );
         Field.SetProperty( "Value", Variant( tick.ask ) );
         
         Field = Fields.GetProperty( "Item", varBid );
         Field.SetProperty( "Value", Variant( tick.bid ) );
         
         Field = Fields.GetProperty( "Item", varLast );
         Field.SetProperty( "Value", Variant( tick.last ) );
         
         Field = Fields.GetProperty( "Item", varTime_msc );
         Field.SetProperty( "Value", Variant( tick.time_msc ) );
         
         Field = Fields.GetProperty( "Item", varFlags );
         Field.SetProperty( "Value", Variant( tick.flags ) );
         
         Recordset.Function( "Update" );
      }
      else {
         Print( Fields.toErrorString() );
      }
   }
}
//+-----------------------------------------------------------------------------------------------+
void PrintError( const ComObject& object )
{
   ComObject Errors = object.GetProperty( "Errors" );
   Variant varErrs = Errors.GetProperty( "Count" );
   int Errs = varErrs.toInt();
   if( Errs ){
      ComObject error = Errors.GetProperty( "Item", Variant(0) );
      Variant varDescription = error.GetProperty( "Description" );
      string strDescription = varDescription.toString();
      Print( strDescription );
   }
   else {
      Print( "Unknown error!" );
   }
}
//+-----------------------------------------------------------------------------------------------+
